"""
Plot histograms of Waiting Time and Post Bunkering Time
Created: 26. April 2018
Author: Marion Aarsnes
"""

import pandas as pd                 # Read CSV
import seaborn as sns               # Library of plots built upon matplotlib
import matplotlib.pyplot as plt     # Plots
import matplotlib.ticker as mtick   # Formatting plots

# Read CSV file
AIS_dataframe = pd.read_csv('filedirectory/filename.csv')
print(AIS_dataframe.columns)

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.s', 'length.s', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'BunkUnixStart',
       'BunkUnixEnd', 'AnchorUnixStart', 'AnchorUnixEnd', 'confidence_mean', 'bunkeringType', 'timeBeforeBunkering_hrs', 'timeAfterBunkering_hrs']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.s": "ShipType",
                                              "length.s": "ShipLength",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs",
                                              "confidence_mean": "ConfMean",
                                              "bunkeringType": "BunkeringType",
                                              "timeBeforeBunkering_hrs": "WaitingTime",
                                              "timeAfterBunkering_hrs": "PBTime"})

AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]
# Removing rows with Confidence Mean < 0.7
AIS_dataframe = AIS_dataframe[AIS_dataframe.ConfMean >= 0.7]

# Reduce dataframe to shiptypes with minimum 100 registrations
AIS_dataframe = AIS_dataframe.query("ShipType in ('Bulk Carrier','Container Ship','Crude Oil Tanker','Chemical/Oil Products Tanker',"
                                    "'Vehicles Carrier','LPG Tanker','General Cargo Ship','Oil Products Tanker',"
                                    "'Ore Carrier','LNG Tanker',)")

# Create new dataframes for bunkering type
# Anchorage
AIS_dataframe_anchor = AIS_dataframe[AIS_dataframe.BunkeringType == 'Anchorage']
# Berth
AIS_dataframe_berth = AIS_dataframe[AIS_dataframe.BunkeringType == 'Berth']

# Set plot stylr
sns.set(style="whitegrid", color_codes=True)

# ------------------ Plot WT vs PBT ------------------

# Scatter plot showing waiting time vs anchorage time after bunkering
plt.scatter(AIS_dataframe.WaitingTime, AIS_dataframe.PBTime)
plt.xlim(-10,200)
plt.ylim(-10,200)
#plt.title('Waiting time vs time after bunkering')
plt.ylabel("Post Bunkering Time [hours]")
plt.xlabel("Waiting Time [hours]")
plt.show()

grid2 = sns.jointplot(AIS_dataframe.WaitingTime, AIS_dataframe.PBTime, kind="reg", joint_kws={'line_kws':{'color':'y'}})
grid2.set_axis_labels("Waiting Time [hours]","Post Bunkering Time [hours]")
plt.xlim(-10,200)
plt.ylim(-10,200)
plt.show()

# ---------------- Define bin size and number of bins ----------------------

# All operations
b_len = 2.5 # Bin size
min_wall = min(AIS_dataframe['WaitingTime'])
max_wall = max(AIS_dataframe['WaitingTime'])
bsize_wall = int((max_wall-min_wall)/b_len) # Number of bins must be an integer

# At anchorage
b_len = 2.5 # Bin size
min_wan = min(AIS_dataframe_anchor['WaitingTime'])
max_wan = max(AIS_dataframe_anchor['WaitingTime'])
bsize_wan = int((max_wan-min_wan)/b_len) # Number of bins must be an integer

# At berth
b_len = 2.5 # Bin size
min_wber = min(AIS_dataframe_berth['WaitingTime'])
max_wber = max(AIS_dataframe_berth['WaitingTime'])
bsize_wber = int((max_wber-min_wber)/b_len) # Number of bins must be an integer


# ------------------- Waiting Time ---------------------
# Plot Waiting Time for all operations
plt.figure()
ax = sns.distplot(AIS_dataframe['WaitingTime'],
                    bins=bsize_wall,kde=False, hist_kws=dict(edgecolor="b", linewidth=1)) # kde=False, color="b", 112
ax.set(ylabel="Number of operations", xlabel="Waiting time [hours]", title ='All')
plt.xlim(0,25)
plt.legend()
plt.show()

# Plot Waiting Time at anchorage
plt.figure()
ax = sns.distplot(AIS_dataframe_anchor['WaitingTime'],
                    bins=bsize_wan,kde=False, hist_kws=dict(edgecolor="b", linewidth=1)) # kde=False, color="b", 112
ax.set(ylabel="Number of operations", xlabel="Waiting time [hours]", title ='At anchorage')
plt.xlim(0,25)
plt.legend()
plt.show()

# Plot Waiting Time at berth
plt.figure()
ax = sns.distplot(AIS_dataframe_berth['WaitingTime'],
                     bins=bsize_wber,kde=False, hist_kws=dict(edgecolor="b", linewidth=1)) # kde=False, color="b",
ax.set(ylabel="Number of operations", xlabel="Waiting time [hours]", title ='At berth')
plt.xlim(0,25)
plt.legend()
plt.show()


# --------------- Post Bunkering Time ---------------

# Define bin size and number of bins

# All operations
b_lenp = 2.5 # Bin size
min_pall = min(AIS_dataframe['PBTime'])
max_pall = max(AIS_dataframe['PBTime'])
bsize_pall = int((max_pall-min_pall)/b_lenp) # Number of bins must be an integer

# At anchorage
b_lenp = 2.5 # Bin size
min_pan = min(AIS_dataframe_anchor['PBTime'])
max_pan = max(AIS_dataframe_anchor['PBTime'])
bsize_pan = int((max_pan-min_pan)/b_lenp) # Number of bins must be an integer

# At berth
b_lenp = 2.5 # Bin size
min_pber = min(AIS_dataframe_berth['PBTime'])
max_pber = max(AIS_dataframe_berth['PBTime'])
bsize_pber = int((max_pber-min_pber)/b_lenp) # Number of bins must be an integer

# Plot Post Bunkering Time for all operations
plt.figure()
ax = sns.distplot(AIS_dataframe['PBTime'],
                    bins=bsize_pall,kde=False, hist_kws=dict(edgecolor="b", linewidth=1)) # kde=False, color="b", 112
ax.set(ylabel="Number of operations", xlabel="Post Bunkering Time [hours]", title ='All')
plt.xlim(0,25)
plt.legend()
plt.show()

# Plot Post Bunkering Time at anchorage
plt.figure()
ax = sns.distplot(AIS_dataframe_anchor['PBTime'],
                    bins=bsize_pan,kde=False, hist_kws=dict(edgecolor="b", linewidth=1)) # kde=False, color="b", 112
ax.set(ylabel="Number of operations", xlabel="Post Bunkering Time [hours]", title ='At anchorage')
plt.xlim(0,25)
plt.legend()
plt.show()

# Plot Post Bunkering Time at berth
plt.figure()
ax = sns.distplot(AIS_dataframe_berth['PBTime'],
                     bins=bsize_pber,kde=False, hist_kws=dict(edgecolor="b", linewidth=1)) # kde=False, color="b",
ax.set(ylabel="Number of operations", xlabel="Post Bunkering Time [hours]", title ='At berth')
plt.xlim(0,25)
plt.legend()
plt.show()