"""
Plot different shiptypes in histograms
Created: 11. April 2018
Author: Marion Aarsnes
"""

# Import packages
import pandas as pd                 # Read CSV
import seaborn as sns               # Library of plots built upon matplotlib
import matplotlib.pyplot as plt     # Plots
import matplotlib.ticker as mtick   # Formatting plots
import numpy as np                  # Scientific calculation

# Read CSV file
AIS_dataframe = pd.read_csv('filedirectory/filename.csv')
print(AIS_dataframe.columns)

# Clean up dataframe
AIS_dataframe = AIS_dataframe[['imo.b', 'shipType.s', 'length.s', 'bunkeringLength_hrs', 'anchoringLength_hrs', 'confidence_mean']]
AIS_dataframe = AIS_dataframe.rename(columns={"imo.b": "IMO",
                                              "shipType.s": "ShipType",
                                              "length.s": "ShipLength",
                                              "anchoringLength_hrs": "AnchoringLengthHrs",
                                              "bunkeringLength_hrs": "BunkeringLengthHrs",
                                              "confidence_mean": "ConfMean"})

AIS_dataframe = AIS_dataframe.fillna(value="nan")[AIS_dataframe.ShipType != "nan"]
# Reduce dataframe to shiptypes with minimum 100 registrations
AIS_dataframe = AIS_dataframe[AIS_dataframe.ConfMean >= 0.7]

# Print top 5 lines of dataframe
print(AIS_dataframe.head(5))
# List all unique shiptypes
print(AIS_dataframe['ShipType'].unique())

# Set plot variables
sns.set(style="whitegrid", color_codes=True)
sns.despine(left=True)

# -------------------- Anchorage lasting longer than value a -----------------------

# Create new dataframe for longer lasting anchorages
a = 150
b = 35

# Create column with same value for all operations lasting longer than value a
def fill_values(a):
    anch_len_x = []
    bunk_len_x = []
    for i in range(len(AIS_dataframe.AnchoringLengthHrs)):
        anch_len_x.append(a)
        bunk_len_x.append(b)
    return anch_len_x, bunk_len_x

anch_len_x, bunk_len_x = fill_values(a)


# Create column index to dataframe
AIS_dataframe['AnchorLengthx'] = AIS_dataframe.index
AIS_dataframe['BunkLengthx'] = AIS_dataframe.index
# Insert new data to dataframe
AIS_dataframe.loc[:,'AnchorLengthx'] = pd.Series(anch_len_x, index=AIS_dataframe.index)
AIS_dataframe.loc[:,'BunkLengthx'] = pd.Series(bunk_len_x, index=AIS_dataframe.index)

# Create new dataframe for operations lasting longer than value a
AIS_dataframe_x = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs >= a]

# Plot tankships, oil products tanker, bunkering tanker and cargo ships in one figure

# Reduce dataframe to contain only operations lasting shorter than value a
AIS_dataframe = AIS_dataframe[AIS_dataframe.AnchoringLengthHrs < a]
# Reduce dataframe to contain specified shiptypes
AIS_dataframe = AIS_dataframe.query("ShipType in ('Container Ship', 'Bulk Carrier', 'Crude Oil Tanker')")


# Specify plotting colors
colors = ['b', 'g', 'r', 'k']


# Plot laying time for shiptypes in one figure
i = 0
for shiptype in AIS_dataframe['ShipType'].unique():
    ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == shiptype]['AnchoringLengthHrs'],
                      hist_kws=dict(edgecolor=colors[i], linewidth=2), label=str(shiptype))
    i +=1
axa = sns.distplot(AIS_dataframe_x['AnchorLengthx'],
                     kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage of more than 180 hours')
ax.set(ylabel="Density", xlabel="Anchoring length [hours]", title="Anchorage time")
plt.legend()
plt.show()


# -------------- Plot bunkering time for ship types in one figure --------------

# Laying time
# Bin size
bin_widthl = 5

# Define number of bins
min_optl = min(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['AnchoringLengthHrs'])
max_optl = max(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['AnchoringLengthHrs'])
bsize_optl = int((max_optl-min_optl)/bin_widthl)    # Number of bins must be an integer
min_gcsl = min(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['AnchoringLengthHrs'])
max_gcsl = max(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['AnchoringLengthHrs'])
bsize_gcsl = int((max_gcsl-min_gcsl)/bin_widthl)    # Number of bins must be an integer
min_ltl = min(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['AnchoringLengthHrs'])
max_ltl = max(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['AnchoringLengthHrs'])
bsize_ltl = int((max_ltl-min_ltl)/bin_widthl)       # Number of bins must be an integer


# Plot laying time
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['AnchoringLengthHrs'],
                    bins=bsize_optl, color='g', hist_kws=dict(edgecolor='g', linewidth=2), label='Oil Products Tanker')
ax1 = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['AnchoringLengthHrs'],
                    bins=bsize_gcsl, color='r', hist_kws=dict(edgecolor='r', linewidth=2), label='General Cargo Ship')
ax2 = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['AnchoringLengthHrs'],
                    bins=bsize_ltl, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='LNG Tanker')
axb = sns.distplot(AIS_dataframe_x['AnchorLengthx'],
                    kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage of more than 180 hours')
ax.set(ylabel="Density", xlabel="Lay time [hours]", title="Lay time")
plt.legend()
plt.xlim(0,80)
plt.show()

# Bunkering time
# Bin size
bin_width = 1

# Define number of bins
min_opt = min(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['BunkeringLengthHrs'])
max_opt = max(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['BunkeringLengthHrs'])
bsize_opt = int((max_opt-min_opt)/bin_width)    # Number of bins must be an integer
min_gcs = min(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['BunkeringLengthHrs'])
max_gcs = max(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['BunkeringLengthHrs'])
bsize_gcs = int((max_gcs-min_gcs)/bin_width)    # Number of bins must be an integer
min_lt = min(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['BunkeringLengthHrs'])
max_lt = max(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['BunkeringLengthHrs'])
bsize_lt = int((max_lt-min_lt)/bin_width)       # Number of bins must be an integer


# Plot bunkering length
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['BunkeringLengthHrs'],
                    bins=bsize_opt, color='g', hist_kws=dict(edgecolor='g', linewidth=2), label='Oil Products Tanker')
ax1 = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['BunkeringLengthHrs'],
                    bins=bsize_gcs, color='r', hist_kws=dict(edgecolor='r', linewidth=2), label='General Cargo Ship')
ax2 = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['BunkeringLengthHrs'],
                    bins=bsize_lt, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='LNG Tanker')
#axb = sns.distplot(AIS_dataframe_x['BunkLengthx'],
                    #kde = False, color = '.2',  hist_kws=dict(edgecolor="k", linewidth=3), label='Anchorage of more than 180 hours')
ax.set(ylabel="Density", xlabel="Bunkering time [hours]", title="Bunkering time")
plt.legend()
plt.xlim(0,35)
plt.show()

# Separate plots
# Container ship
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['BunkeringLengthHrs'],
                    bins=bsize_opt, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='Container Ship')
ax.set(ylabel="Density", xlabel="Bunkering time [hours]", title="Bunkering time Container Ship")
plt.xlim(0,30)
plt.legend()
plt.show()

# Bulk carrier
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['BunkeringLengthHrs'],
                    bins=bsize_gcs, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='Bulk Carrier')
ax.set(ylabel="Density", xlabel="Bunkering time [hours]", title="Bunkering time Bulk Carrier")
plt.xlim(0,30)
plt.legend()
plt.show()

# Crude oil tanker
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['BunkeringLengthHrs'],
                    bins=bsize_lt, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='Crude Oil Tanker')
ax.set(ylabel="Density", xlabel="Bunkering time [hours]", title="Bunkering time Crude Oil Tanker")
plt.xlim(0,30)
plt.legend()
plt.show()


# --------------- Plot shiplengths per shiptypes -------------------

# Bin size
bin_widthl = 10

# Define number of bins
min_optl = min(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['ShipLength'])
max_optl = max(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['ShipLength'])
bsize_optl = int((max_optl-min_optl)/bin_widthl)    # Number of bins must be an integer
min_gcsl = min(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['ShipLength'])
max_gcsl = max(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['ShipLength'])
bsize_gcsl = int((max_gcsl-min_gcsl)/bin_widthl)    # Number of bins must be an integer
min_ltl = min(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['ShipLength'])
max_ltl = max(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['ShipLength'])
bsize_ltl = int((max_ltl-min_ltl)/bin_widthl)       # Number of bins must be an integer


# Plot histograms

# Container ship
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Container Ship']['ShipLength'],
                    bins=bsize_optl, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='Container Ship')
ax.set(ylabel="Density", xlabel="Ship length [meters]", title="Shiplength Container Ship")
plt.xlim(100,450)
plt.legend()
plt.show()

# Bulk carrier
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Bulk Carrier']['ShipLength'],
                    bins=bsize_gcsl, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='Bulk Carrier')
ax.set(ylabel="Density", xlabel="Ship length [meters]", title="Shiplength Bulk Carrier")
plt.xlim(100,450)
plt.legend()
plt.show()

# Crude oil tanker
ax = sns.distplot(AIS_dataframe[AIS_dataframe.ShipType == 'Crude Oil Tanker']['ShipLength'],
                    bins=bsize_ltl, color='b', hist_kws=dict(edgecolor='b', linewidth=2), label='Crude Oil Tanker')
ax.set(ylabel="Density", xlabel="Ship length [meters]", title="Shiplength Crude Oil Tanker")
plt.xlim(100,450)
plt.legend()
plt.show()




